#pragma once

// UrsStringHelper.h

#include <Arduino.h>
#include <concepts>

class UrsStringHelperClass {
public:
   static void setThousandSeparator(char sepChar) { thousandSeparator[0] = sepChar; }; // Default is ','.
   static void setDecimalSeparator(char sepChar) { decimalSeparator[0] = sepChar; }  // Default is '.'.
   static void setHexSeparator(char sepChar) { hexSeparator[0] = sepChar; }  // Default is ' '.


   // Insert thousands separators to uint64_t numbers.
   static String valToStr(uint64_t val);

   // Template for other signed number types
   template <typename T>
   static inline String valToStr(T val) requires std::unsigned_integral<T> { return valToStr(static_cast<uint64_t>(val)); }


   // Insert thousands separators to int64_t numbers.
   static String valToStr(int64_t val, bool plusSign = false);

   // Template for other signed number types
   template <typename T>
   static inline String valToStr(T val) requires std::signed_integral<T> { return valToStr(static_cast<int64_t>(val)); }

   // Get Hex value with leading zeroes and separator each four digits
   static inline String valToHex(uint64_t val) { return valToHex(val, 16); }
   static inline String valToHex(uint32_t val) { return valToHex(val, 8); }
   static inline String valToHex(uint16_t val) { return valToHex(val, 4); }
   static inline String valToHex(uint8_t val) { return valToHex(val, 2); }

   static inline String valToHex(int64_t val) { return valToHex(*reinterpret_cast<uint64_t *>( &val)); }
   static inline String valToHex(int32_t val) { return valToHex(*reinterpret_cast<uint32_t*>(&val)); }
   static inline String valToHex(int16_t val) { return valToHex(*reinterpret_cast<uint16_t*>(&val)); }
   static inline String valToHex(int8_t val) { return valToHex(*reinterpret_cast<uint8_t*>(&val)); }


      // Reverse the characters in a string
   static void strrev(String& string);

protected:
   static char thousandSeparator[];
   static char decimalSeparator[];
   static char hexSeparator[];
   static String valToStr(String header, uint64_t x);
   static String valToHex(uint64_t val, uint8_t digits);

};

extern UrsStringHelperClass UrsStringHelper;
extern UrsStringHelperClass& SHelper;